"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFirestoreChangeValue = void 0;
function parseFirestoreChangeValue(idParameter, change, event) {
    // Before value is an empty object on creation (not null)
    if (change && Object.keys(change).length) {
        const value = parseFirestoreMapValue(change);
        const id = event.params[idParameter];
        if (!id) {
            console.error(event, change);
            throw new Error('Unable to find document id for' + event.resource);
        }
        return {
            id: id,
            createdAt: new Date(change.createTime),
            updatedAt: new Date(change.updateTime),
            data: value
        };
    }
    return null;
}
exports.parseFirestoreChangeValue = parseFirestoreChangeValue;
function parseFirestoreValue(value) {
    if (value.mapValue) {
        return parseFirestoreMapValue(value.mapValue);
    }
    else if (value.arrayValue) {
        if (value.arrayValue.values) {
            return value.arrayValue.values.map((value) => {
                return parseFirestoreValue(value);
            });
        }
        else {
            return [];
        }
    }
    else if (value.timestampValue) {
        return new Date(value.timestampValue);
    }
    else if (value.integerValue) {
        return parseInt(value.integerValue, 10);
    }
    return Object.values(value)[0] || null;
}
function parseFirestoreMapValue(value) {
    const obj = {};
    const keys = Object.keys(value.fields || {});
    for (const key of keys) {
        const val = value.fields[key];
        obj[key] = parseFirestoreValue(val);
    }
    return obj;
}
//# sourceMappingURL=utils.js.map